@@ -2,11 +2,6 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-/// Note: the VM concatenates all patch files into a single patch file. This
-/// file is the first patch in "dart:isolate" which contains all the imports
-/// used by patches of that library. We plan to change this when we have a
-/// shared front end and simply use parts.
-
 import "dart:_internal" show ClassID, VMLibraryHooks, patch;
 
 import "dart:async"
@@ -16,7 +11,7 @@ import "dart:collection" show HashMap;
 import "dart:typed_data" show ByteBuffer, TypedData, Uint8List;
 
 /// These are the additional parts of this patch library:
-// part "timer_impl.dart";
+part "timer_impl.dart";
 
 @patch
 class ReceivePort {
@@ -31,27 +26,28 @@ class ReceivePort {
 }
 
 @patch
+@pragma("vm:entry-point")
 class Capability {
   @patch
-  factory Capability() => new _CapabilityImpl();
+  factory Capability() => new _Capability();
 }
 
 @pragma("vm:entry-point")
-class _CapabilityImpl implements Capability {
-  @pragma("vm:external-name", "CapabilityImpl_factory")
-  external factory _CapabilityImpl();
+class _Capability implements Capability {
+  @pragma("vm:external-name", "Capability_factory")
+  external factory _Capability();
 
   bool operator ==(Object other) {
-    return (other is _CapabilityImpl) && _equals(other);
+    return (other is _Capability) && _equals(other);
   }
 
   int get hashCode {
     return _get_hashcode();
   }
 
-  @pragma("vm:external-name", "CapabilityImpl_equals")
+  @pragma("vm:external-name", "Capability_equals")
   external bool _equals(Object other);
-  @pragma("vm:external-name", "CapabilityImpl_get_hashcode")
+  @pragma("vm:external-name", "Capability_get_hashcode")
   external int _get_hashcode();
 }
 
@@ -66,13 +62,13 @@ class RawReceivePort {
    */
   @patch
   factory RawReceivePort([Function? handler, String debugName = '']) {
-    _RawReceivePortImpl result = new _RawReceivePortImpl(debugName);
+    _RawReceivePort result = new _RawReceivePort(debugName);
     result.handler = handler;
     return result;
   }
 }
 
-class _ReceivePortImpl extends Stream implements ReceivePort {
+final class _ReceivePortImpl extends Stream implements ReceivePort {
   _ReceivePortImpl([String debugName = ''])
       : this.fromRawReceivePort(new RawReceivePort(null, debugName));
 
@@ -131,30 +127,28 @@ Function _getIsolateScheduleImmediateClosure() {
 }
 
 @pragma("vm:entry-point")
-class _RawReceivePortImpl implements RawReceivePort {
-  factory _RawReceivePortImpl(String debugName) {
-    final port = _RawReceivePortImpl._(debugName);
-    _portMap[port._get_id()] = <String, dynamic>{
-      'port': port,
-    };
+final class _RawReceivePort implements RawReceivePort {
+  factory _RawReceivePort(String debugName) {
+    final port = _RawReceivePort._(debugName);
+    _portMap[port._get_id()] = port;
     return port;
   }
 
-  @pragma("vm:external-name", "RawReceivePortImpl_factory")
-  external factory _RawReceivePortImpl._(String debugName);
+  @pragma("vm:external-name", "RawReceivePort_factory")
+  external factory _RawReceivePort._(String debugName);
 
   close() {
     // Close the port and remove it from the handler map.
     _portMap.remove(this._closeInternal());
   }
 
-  SendPort get sendPort {
-    return _get_sendport();
-  }
+  @pragma("vm:recognized", "other")
+  @pragma("vm:prefer-inline")
+  @pragma("vm:external-name", "RawReceivePort_getSendPort")
+  external SendPort get sendPort;
 
   bool operator ==(var other) {
-    return (other is _RawReceivePortImpl) &&
-        (this._get_id() == other._get_id());
+    return (other is _RawReceivePort) && (this._get_id() == other._get_id());
   }
 
   int get hashCode {
@@ -162,27 +156,25 @@ class _RawReceivePortImpl implements RawReceivePort {
   }
 
   /**** Internal implementation details ****/
-  @pragma("vm:external-name", "RawReceivePortImpl_get_id")
+  @pragma("vm:external-name", "RawReceivePort_get_id")
   external int _get_id();
-  @pragma("vm:external-name", "RawReceivePortImpl_get_sendport")
-  external SendPort _get_sendport();
 
   // Called from the VM to retrieve the handler for a message.
   @pragma("vm:entry-point", "call")
   static _lookupHandler(int id) {
-    var result = _portMap[id]?['handler'];
-    return result;
+    return _portMap[id]?._handler;
   }
 
+  // Called from the VM service to enumerate ports.
   @pragma("vm:entry-point", "call")
   static _lookupOpenPorts() {
-    return _portMap.values.map((e) => e['port']).toList();
+    return _portMap.values.toList();
   }
 
-  // Called from the VM to retrieve  the handler and handle a message.
+  // Called from the VM to dispatch a message.
   @pragma("vm:entry-point", "call")
   static _handleMessage(int id, var message) {
-    final handler = _portMap[id]?['handler'];
+    final Function? handler = _portMap[id]?._handler;
     if (handler == null) {
       return null;
     }
@@ -195,32 +187,41 @@ class _RawReceivePortImpl implements RawReceivePort {
   }
 
   // Call into the VM to close the VM maintained mappings.
-  @pragma("vm:external-name", "RawReceivePortImpl_closeInternal")
+  @pragma("vm:external-name", "RawReceivePort_closeInternal")
   external int _closeInternal();
 
   // Set this port as active or inactive in the VM. If inactive, this port
   // will not be considered live even if it hasn't been explicitly closed.
-  // TODO(bkonyi): determine if we want to expose this as an option through
-  // RawReceivePort.
-  @pragma("vm:external-name", "RawReceivePortImpl_setActive")
-  external _setActive(bool active);
+  @pragma("vm:external-name", "RawReceivePort_setActive")
+  external void _setActive(bool active);
+  @pragma("vm:external-name", "RawReceivePort_getActive")
+  external bool _getActive();
+
+  @pragma("vm:recognized", "other")
+  @pragma("vm:prefer-inline")
+  @pragma("vm:external-name", "RawReceivePort_getHandler")
+  external Function? get _handler;
+  @pragma("vm:recognized", "other")
+  @pragma("vm:prefer-inline")
+  @pragma("vm:external-name", "RawReceivePort_setHandler")
+  external set _handler(Function? handler);
 
   void set handler(Function? value) {
-    final int id = this._get_id();
-    if (!_portMap.containsKey(id)) {
-      _portMap[id] = <String, dynamic>{
-        'port': this,
-      };
-    }
-    _portMap[id]!['handler'] = value;
+    _handler = value;
   }
 
-  static final _portMap = <int, Map<String, dynamic>>{};
+  void set keepIsolateAlive(bool value) {
+    _setActive(value);
+  }
+
+  bool get keepIsolateAlive => _getActive();
+
+  static final _portMap = <int, _RawReceivePort>{};
 }
 
 @pragma("vm:entry-point")
-class _SendPortImpl implements SendPort {
-  factory _SendPortImpl._uninstantiable() {
+final class _SendPort implements SendPort {
+  factory _SendPort._uninstantiable() {
     throw "Unreachable";
   }
 
@@ -231,7 +232,7 @@ class _SendPortImpl implements SendPort {
   }
 
   bool operator ==(var other) {
-    return (other is _SendPortImpl) && (this._get_id() == other._get_id());
+    return (other is _SendPort) && (this._get_id() == other._get_id());
   }
 
   int get hashCode {
@@ -239,17 +240,16 @@ class _SendPortImpl implements SendPort {
   }
 
   /*--- private implementation ---*/
-  @pragma("vm:external-name", "SendPortImpl_get_id")
+  @pragma("vm:external-name", "SendPort_get_id")
   external _get_id();
-  @pragma("vm:external-name", "SendPortImpl_get_hashcode")
+  @pragma("vm:external-name", "SendPort_get_hashcode")
   external _get_hashcode();
 
   // Forward the implementation of sending messages to the VM.
-  @pragma("vm:external-name", "SendPortImpl_sendInternal_")
+  @pragma("vm:external-name", "SendPort_sendInternal_")
   external void _sendInternal(var message);
 }
 
-typedef _NullaryFunction();
 typedef _UnaryFunction(Never args);
 typedef _BinaryFunction(Never args, Never message);
 
@@ -290,9 +290,9 @@ void _delayEntrypointInvocation(Function entryPoint, List<String>? args,
     port.close();
     if (allowZeroOneOrTwoArgs) {
       if (entryPoint is _BinaryFunction) {
-        (entryPoint as dynamic)(args, message);
+        (entryPoint as Function)(args, message);
       } else if (entryPoint is _UnaryFunction) {
-        (entryPoint as dynamic)(args);
+        (entryPoint as Function)(args);
       } else {
         entryPoint();
       }
@@ -304,7 +304,7 @@ void _delayEntrypointInvocation(Function entryPoint, List<String>? args,
 }
 
 @patch
-class Isolate {
+final class Isolate {
   static final _currentIsolate = _getCurrentIsolate();
   static final _rootUri = _getCurrentRootUri();
 
@@ -315,13 +315,13 @@ class Isolate {
   String? get debugName => _getDebugName(controlPort);
 
   @patch
-  static Future<Uri?> get packageRoot {
-    return Future.value(null);
+  static Future<Uri?> get packageConfig {
+    return Future.value(packageConfigSync);
   }
 
   @patch
-  static Future<Uri?> get packageConfig {
-    var hook = VMLibraryHooks.packageConfigUriFuture;
+  static Uri? get packageConfigSync {
+    var hook = VMLibraryHooks.packageConfigUriSync;
     if (hook == null) {
       throw new UnsupportedError("Isolate.packageConfig");
     }
@@ -330,16 +330,21 @@ class Isolate {
 
   @patch
   static Future<Uri?> resolvePackageUri(Uri packageUri) {
-    var hook = VMLibraryHooks.resolvePackageUriFuture;
+    return Future.value(resolvePackageUriSync(packageUri));
+  }
+
+  @patch
+  static Uri? resolvePackageUriSync(Uri packageUri) {
+    var hook = VMLibraryHooks.resolvePackageUriSync;
     if (hook == null) {
-      throw new UnsupportedError("Isolate.resolvePackageUri");
+      throw new UnsupportedError("Isolate.resolvePackageUriSync");
     }
     return hook(packageUri);
   }
 
   static bool _packageSupported() =>
-      (VMLibraryHooks.packageConfigUriFuture != null) &&
-      (VMLibraryHooks.resolvePackageUriFuture != null);
+      (VMLibraryHooks.packageConfigUriSync != null) &&
+      (VMLibraryHooks.resolvePackageUriSync != null);
 
   @patch
   static Future<Isolate> spawn<T>(void entryPoint(T message), T message,
@@ -348,13 +353,7 @@ class Isolate {
       SendPort? onExit,
       SendPort? onError,
       String? debugName}) async {
-    // `paused` isn't handled yet.
-    // Check for the type of `entryPoint` on the spawning isolate to make
-    // error-handling easier.
-    if (entryPoint is! _UnaryFunction) {
-      throw new ArgumentError(entryPoint);
-    }
-    // The VM will invoke [_startIsolate] with entryPoint as argument.
+    // The VM will invoke [_startIsolate] with [entryPoint] as argument.
 
     // We do not inherit the package config settings from the parent isolate,
     // instead we use the values that were set on the command line.
@@ -369,7 +368,7 @@ class Isolate {
       if (Isolate._packageSupported()) {
         // resolving script uri is not really necessary, but can be useful
         // for better failed-to-lookup-function-in-a-script spawn errors.
-        script = await Isolate.resolvePackageUri(script);
+        script = Isolate.resolvePackageUriSync(script);
       }
     }
 
@@ -418,12 +417,12 @@ class Isolate {
     if (automaticPackageResolution) {
       if (packageRoot != null) {
         throw new ArgumentError("Cannot simultaneously request "
-            "automaticPackageResolution and specify a"
+            "automaticPackageResolution and specify a "
             "packageRoot.");
       }
       if (packageConfig != null) {
         throw new ArgumentError("Cannot simultaneously request "
-            "automaticPackageResolution and specify a"
+            "automaticPackageResolution and specify a "
             "packageConfig.");
       }
     } else {
@@ -438,7 +437,7 @@ class Isolate {
     // Inherit this isolate's package resolution setup if not overridden.
     if (!automaticPackageResolution && packageConfig == null) {
       if (Isolate._packageSupported()) {
-        packageConfig = await Isolate.packageConfig;
+        packageConfig = Isolate.packageConfigSync;
       }
     }
 
@@ -447,7 +446,7 @@ class Isolate {
       // Avoid calling resolvePackageUri if not strictly necessary in case
       // the API is not supported.
       if (packageConfig.isScheme("package")) {
-        packageConfig = await Isolate.resolvePackageUri(packageConfig);
+        packageConfig = Isolate.resolvePackageUriSync(packageConfig);
       }
     }
 
@@ -456,6 +455,7 @@ class Isolate {
 
     final RawReceivePort readyPort =
         new RawReceivePort(null, 'Isolate.spawnUri ready');
+    final enableAsserts = checked;
     try {
       _spawnUri(
           readyPort.sendPort,
@@ -466,7 +466,7 @@ class Isolate {
           onExit,
           onError,
           errorsAreFatal,
-          checked,
+          enableAsserts,
           null,
           /* environment */
           packageConfigString,
@@ -527,7 +527,7 @@ class Isolate {
       SendPort? onExit,
       SendPort? onError,
       bool errorsAreFatal,
-      bool? checked,
+      bool? enableAsserts,
       List? environment,
       String? packageConfig,
       String? debugName);
@@ -536,7 +536,7 @@ class Isolate {
   external static void _sendOOB(port, msg);
 
   @pragma("vm:external-name", "Isolate_getDebugName")
-  external static String _getDebugName(SendPort controlPort);
+  external static String? _getDebugName(SendPort controlPort);
 
   @patch
   void _pause(Capability resumeCapability) {
@@ -590,7 +590,7 @@ class Isolate {
   }
 
   @patch
-  void kill({int priority: beforeNextEvent}) {
+  void kill({int priority = beforeNextEvent}) {
     var msg = new List<Object?>.filled(4, null)
       ..[0] = 0 // Make room for OOB message type.
       ..[1] = _KILL
@@ -601,7 +601,7 @@ class Isolate {
 
   @patch
   void ping(SendPort responsePort,
-      {Object? response, int priority: immediate}) {
+      {Object? response, int priority = immediate}) {
     var msg = new List<Object?>.filled(5, null)
       ..[0] = 0 // Make room for OOM message type.
       ..[1] = _PING
@@ -653,7 +653,14 @@ class Isolate {
   @pragma("vm:external-name", "Isolate_exit_")
   external static Never _exit(SendPort? finalMessagePort, Object? message);
 
+  @pragma("vm:entry-point")
+  static bool _mayExit = true;
+
+  @patch
   static Never exit([SendPort? finalMessagePort, Object? message]) {
+    if (!_mayExit) {
+      throw UnsupportedError("Isolate.exit");
+    }
     _exit(finalMessagePort, message);
   }
 
@@ -684,7 +691,8 @@ class Isolate {
 }
 
 @patch
-abstract class TransferableTypedData {
+@pragma("vm:entry-point")
+abstract final class TransferableTypedData {
   @patch
   factory TransferableTypedData.fromList(List<TypedData> chunks) {
     if (chunks == null) {
@@ -701,7 +709,7 @@ abstract class TransferableTypedData {
 }
 
 @pragma("vm:entry-point")
-class _TransferableTypedDataImpl implements TransferableTypedData {
+final class _TransferableTypedDataImpl implements TransferableTypedData {
   @pragma("vm:external-name", "TransferableTypedData_factory")
   external factory _TransferableTypedDataImpl(List<TypedData> list);
 
